Perm

Struct Perm 

pub struct Perm { /* private fields */ }
A permutation of 0..n.

Stored as Either<usize, Vec<usize>>: Left(n) is the identity permutation on 0..n, and Right(v) is an explicit image vector where v[i] is the image of i.

Implementations

Source

impl Perm

pub fn from_indices<I>(images: I) -> Self
where I: IntoIterator<Item = usize>,

Create from an iterator of images. The i-th item is the image of i. Panics if the resulting sequence is not a valid permutation.

pub fn id(n: usize) -> Self

The identity permutation on 0..n. Zero-cost.

pub fn len(&self) -> usize

pub fn at(&self, i: usize) -> usize

The image of i.

pub fn is_id(&self) -> bool

true iff this is the identity permutation.

pub fn inv(&self) -> Self

The inverse permutation.

pub fn apply_to<R>(&self, y: Vec<R>) -> Vec<R>

Left group action consuming y: returns a vector result such that result[self.at(i)] = y[i] for every i.

pub fn apply_inv_to<R>(&self, y: Vec<R>) -> Vec<R>

Inverse-left action consuming y: returns a vector result such that result[k] = y[self.at(k)] for every k. Equivalent to self.inv().apply_to(y) but without allocating an inverse permutation.

pub fn shift(self, r: usize) -> Self

Returns a permutation of dimension self.len() + r that is the identity on [0..r) and acts as self (shifted by r) on [r..r + self.len()). Identity is preserved (zero-cost).

pub fn extend(self, r: usize) -> Self

Returns a permutation of dimension self.len() + r that acts as self on [0..self.len()) and the identity on the appended [self.len()..self.len() + r). Identity is preserved (zero-cost).

pub fn forward_indices<I>(n: usize, prefix: I) -> Self
where I: IntoIterator<Item = usize>,

Permutation p of 0..n that sends each index in prefix to a position 0, 1, 2, ... (in the order given), with the remaining indices filling positions in sorted order. If prefix is empty, returns the identity (zero-cost).