pub struct Perm { /* private fields */ }Expand description
A permutation of 0..n.
Stored as Either<usize, Vec<usize>>: Left(n) is the identity
permutation on 0..n, and Right(v) is an explicit image vector
where v[i] is the image of i.
Implementations
Sourceimpl Perm
impl Perm
Sourcepub fn from_indices<I>(images: I) -> Selfwhere
I: IntoIterator<Item = usize>,
pub fn from_indices<I>(images: I) -> Selfwhere
I: IntoIterator<Item = usize>,
Create from an iterator of images. The i-th item is the image of i.
Panics if the resulting sequence is not a valid permutation.
pub fn len(&self) -> usize
Sourcepub fn apply_to<R>(&self, y: Vec<R>) -> Vec<R>
pub fn apply_to<R>(&self, y: Vec<R>) -> Vec<R>
Left group action consuming y: returns a vector result such that
result[self.at(i)] = y[i] for every i.
Sourcepub fn apply_inv_to<R>(&self, y: Vec<R>) -> Vec<R>
pub fn apply_inv_to<R>(&self, y: Vec<R>) -> Vec<R>
Inverse-left action consuming y: returns a vector result such that
result[k] = y[self.at(k)] for every k. Equivalent to
self.inv().apply_to(y) but without allocating an inverse permutation.
Sourcepub fn shift(self, r: usize) -> Self
pub fn shift(self, r: usize) -> Self
Returns a permutation of dimension self.len() + r that is the
identity on [0..r) and acts as self (shifted by r) on
[r..r + self.len()). Identity is preserved (zero-cost).
Sourcepub fn extend(self, r: usize) -> Self
pub fn extend(self, r: usize) -> Self
Returns a permutation of dimension self.len() + r that acts as
self on [0..self.len()) and the identity on the appended
[self.len()..self.len() + r). Identity is preserved (zero-cost).
Sourcepub fn forward_indices<I>(n: usize, prefix: I) -> Selfwhere
I: IntoIterator<Item = usize>,
pub fn forward_indices<I>(n: usize, prefix: I) -> Selfwhere
I: IntoIterator<Item = usize>,
Permutation p of 0..n that sends each index in prefix to a
position 0, 1, 2, ... (in the order given), with the remaining
indices filling positions in sorted order.
If prefix is empty, returns the identity (zero-cost).